window.Gleam.initSearch([{"doc":"optimist","title":"optimist","content":"# optimist\n\n[![Package Version](https://img.shields.io/hexpm/v/optimist)](https://hex.pm/packages/optimist)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/optimist/)\n\n```sh\ngleam add optimist@1\n```\n\nWhen building user interfaces, we are often faced with the question of what to do\nwhile waiting for a response from a server. One common pattern is to update the\nui with an optimistic value - one we expect to get back if an operation is\nsuccessful - and then update the ui again once we get an actual response.\n\nThis package, `optimist`, provides a simple way to manage this pattern through an\n`Optimstic` type and some functions to work with it. Let&#39;s take a quick look at\nwhat you can do...\n\n## 1. Start with a value\n\nFirst things first, we need some value to be optimistic about!\n\n```gleam\nlet wibble = optimist.from(1)\n```\n\n## 2. Update it\n\nWe have two ways to perform optimistic updates on an [`Optimistic`](https://hexdocs.pm/optimist/optimist.html#Optimistic)\nvalue:\n\n- [`push`](https://hexdocs.pm/optimist/optimist.html#push) a new value to replace\n  the current one.\n\n```gleam\nlet wobble = wibble |&gt; optimist.push(2)\n```\n\n- [`update`](https://hexdocs.pm/optimist/optimist.html#update) the current value\n  with a function that takes the current value and returns a new one.\n\n```gleam\nlet wobble = wibble |&gt; optimist.update(int.add(_, 1))\n```\n\n## 3. Do something with it\n\nWe&#39;ve now performed an optimistic update and can render our value in the ui _as\nif the operation has already succeeded_.\n\n```gleam\nhtml.p([], [\n  html.text(&quot;Your value is: &quot;),\n  html.text(optimist.unwrap(wobble) |&gt; int.to_string)\n])\n```\n\n## 4. Resolve the update\n\nWe have a few different options for resolving an optimistic update depending on\nthe situation:\n\n- [`resolve`](https://hexdocs.pm/optimist/optimist.html#resolve) the optimistic\n  update with a new value.\n\n```gleam\nlet response = Ok(2)\nlet resolved = wibble |&gt; optimist.resolve(response)\n```\n\n- [`try`](https://hexdocs.pm/optimist/optimist.html#try) to resolve the\n  optimistic update by applying a function to the response.\n\n```gleam\nlet response = Ok(1)\nlet resolved = wibble |&gt; optimist.try(response, int.add)\n```\n\n- [`revert`](https://hexdocs.pm/optimist/optimist.html#reject) the optimistic\n  update to a previous value.\n\n```gleam\nlet resolved = wibble |&gt; optimist.revert\n```\n\n- [`force`](https://hexdocs.pm/optimist/optimist.html#force) the optimistic\n  update to resolve itself.\n\n```gleam\nlet resolved = wibble |&gt; optimist.force\n```\n","url":"index.html"},{"doc":"optimist","title":"Optimistic","content":"pub opaque type Optimistic(a)\n A value that is either fully resolved or an optimistic update pending resolution.\n You can access the value of an `Optimistic` by using the [`unwrap`](#unwrap)\n function.\n\n\nSynonyms:\noptimist.Optimistic\noptimist Optimistic","url":"optimist.html#Optimistic"},{"doc":"optimist","title":"force","content":"pub fn force(optimistic: Optimistic(a)) -&gt; Optimistic(a)\n Take an `Optimistic` update and force it to be resolved. This will erase the\n fallback value and commit to whatever value is currently stored.\n\n ### Forcing an optimistic update\n\n \n import gleeunit/should\n import optimist\n\n pub fn example() {\n   let optimistic =\n     optimist.from(1)\n     |&gt; optimist.update(2)\n     |&gt; optimist.force\n     |&gt; optimist.unwrap\n\n   optimistic |&gt; should.equal(2)\n }\n \n\nSynonyms:\noptimist.force\noptimist force","url":"optimist.html#force"},{"doc":"optimist","title":"from","content":"pub fn from(value: a) -&gt; Optimistic(a)\n Construct an `Optimistic` update from a fully resolved value.\n\nSynonyms:\noptimist.from\noptimist from","url":"optimist.html#from"},{"doc":"optimist","title":"is_pending","content":"pub fn is_pending(optimistic: Optimistic(a)) -&gt; Bool\n Determine if an `Optimistic` update is still pending resolution.\n\n **Note**: it is uncommon to need this function. The optimistic ui pattern\n typically means we are pretending an operation succeeded before we know for\n sure. If you want to know if some request or other async job is still in\n progress, you probably have that information in a more direct form elsewhere\n in your state!\n\nSynonyms:\noptimist.is_pending\noptimist is_pending","url":"optimist.html#is_pending"},{"doc":"optimist","title":"is_resolved","content":"pub fn is_resolved(optimistic: Optimistic(a)) -&gt; Bool\n Determine if an `Optimistic` update is fully resolved.\n\n **Note**: it is uncommon to need this function. The optimistic ui pattern\n typically means we are pretending an operation succeeded before we know for\n sure. If you want to know if some request or other async job is still in\n progress, you probably have that information in a more direct form elsewhere\n in your state!\n\nSynonyms:\noptimist.is_resolved\noptimist is_resolved","url":"optimist.html#is_resolved"},{"doc":"optimist","title":"push","content":"pub fn push(optimistic: Optimistic(a), value: a) -&gt; Optimistic(a)\n Push an optimistic update. If the current value is already resolved, this\n becomes a new optimistic update and the current value becomes the fallback.\n If the current value is already an optimistic update, the value is updated\n but the fallback remains unchanged.\n\n ### Optimistic update of a resolved value\n\n \n import gleeunit/should\n import optimist\n\n pub fn example() {\n   let optimistic =\n     optimist.from(1)\n     |&gt; optimist.push(2)\n     |&gt; optimist.unwrap\n\n   optimistic |&gt; should.equal(2)\n }\n \n\n ### Multiple optimistic updates\n\n \n import gleeunit/should\n import optimist\n\n pub fn example() {\n   let optimistic =\n     optimist.from(1)\n     |&gt; optimist.push(2)\n     |&gt; optimist.push(3)\n     |&gt; optimist.unwrap\n\n   optimistic |&gt; should.equal(3)\n }\n \n\n ### Rejecting multiple optimistic updates\n\n \n import gleeunit/should\n import optimist\n\n pub fn example() {\n   let optimistic =\n     optimist.from(1)\n     |&gt; optimist.push(2)\n     |&gt; optimist.push(3)\n     |&gt; optimist.revert\n     |&gt; optimist.unwrap\n\n   optimistic |&gt; should.equal(1)\n }\n \n\nSynonyms:\noptimist.push\noptimist push","url":"optimist.html#push"},{"doc":"optimist","title":"resolve","content":"pub fn resolve(\n  optimistic: Optimistic(a),\n  result: Result(a, b),\n) -&gt; Optimistic(a)\n Take a `Result` and use it to resolve an optimistic update. If the result is\n `Ok` that value becomes a new resolved value. If the result is an `Error` and\n an optimistic update is pending, the fallback value is used to resolve the\n update.\n\n \n case result {\n   Ok(value) -&gt; optimist.from(value)\n   Error(_) -&gt; optimist.revert(optimistic)\n }\n \n\n ### Resolving a successful optimistic update\n\n \n import gleeunit/should\n import optimist\n\n pub fn example() {\n   let result = Ok(2)\n   let optimstic =\n     optimist.from(1)\n     |&gt; optimistic.update(2)\n     |&gt; optimist.resolve(result)\n     |&gt; optimist.unwrap\n\n   optimistic |&gt; should.equal(2)\n }\n \n\n ### Rejecting a failed optimistic update\n\n \n import gleeunit/should\n import optimist\n\n pub fn example() {\n   let result = Error(&quot;failed&quot;)\n   let optimistic =\n     optimist.from(1)\n     |&gt; optimistic.update(2)\n     |&gt; optimist.resolve(result)\n     |&gt; optimist.unwrap\n\n   optimistic |&gt; should.equal(1)\n }\n \n\nSynonyms:\noptimist.resolve\noptimist resolve","url":"optimist.html#resolve"},{"doc":"optimist","title":"revert","content":"pub fn revert(optimistic: Optimistic(a)) -&gt; Optimistic(a)\n Take an `Optimistic` update and revert it back to its initial value if it is\n still pending.\n\n ### Reverting an optimistic update\n\n \n import gleeunit/should\n import optimist\n\n pub fn example() {\n   let optimistic =\n     optimist.from(1)\n     |&gt; optimist.update(2)\n     |&gt; optimist.revert\n     |&gt; optimist.unwrap\n\n   optimistic |&gt; should.equal(1)\n }\n \n\nSynonyms:\noptimist.revert\noptimist revert","url":"optimist.html#revert"},{"doc":"optimist","title":"try","content":"pub fn try(\n  optimistic: Optimistic(a),\n  result: Result(b, c),\n  f: fn(a, b) -&gt; a,\n) -&gt; Optimistic(a)\n Take a `Result` and use it to resolve an optimistic update. If the result is\n `Ok` that provided callback is run using either the value of an already-resolved\n optimistic update, or the **fallback value** of a pending update.\n\n ### Resolving a successful optimistic update\n\n \n import gleeunit/should\n import optimist\n\n pub fn example() {\n   let history = [&quot;hey&quot;, &quot;hi&quot;]\n   let result = Ok(&quot;how are you?&quot;)\n   let optimistic =\n     optimist.from(history)\n     |&gt; optimist.update(list.prepend(_, &quot;how are you?&quot;))\n     |&gt; optimist.try(result, list.prepend)\n     |&gt; optimist.unwrap\n\n   optimistic |&gt; should.equal([&quot;how are you?&quot;, &quot;hey&quot;, &quot;hi&quot;]\n }\n \n\n ### Resolving an unsuccessful optimistic update\n\n \n import gleeunit/should\n import optimist\n\n pub fn example() {\n   let history = [&quot;hey&quot;, &quot;hi&quot;]\n   let result = Error(Nil)\n   let optimistic =\n     optimist.from(history)\n     |&gt; optimist.update(list.prepend(_, &quot;how are you?&quot;))\n     |&gt; optimist.try(result, list.prepend)\n     |&gt; optimist.unwrap\n\n   optimistic |&gt; should.equal([&quot;hey&quot;, &quot;hi&quot;]\n }\n \n\nSynonyms:\noptimist.try\noptimist try","url":"optimist.html#try"},{"doc":"optimist","title":"unwrap","content":"pub fn unwrap(optimistic: Optimistic(a)) -&gt; a\n Unwrap an `Optimistic` value and return the underlying value.\n\nSynonyms:\noptimist.unwrap\noptimist unwrap","url":"optimist.html#unwrap"},{"doc":"optimist","title":"update","content":"pub fn update(\n  optimistic: Optimistic(a),\n  f: fn(a) -&gt; a,\n) -&gt; Optimistic(a)\n Perform an optimistic update. If the current value is already resolved, this\n becomes a new optimistic update and the current value becomes the fallback.\n If the current value is already an optimistic update, the value is updated\n but the fallback remains unchanged.\n\n ### Optimistic update of a resolved value\n\n \n import gleeunit/should\n import optimist\n\n pub fn example() {\n   let optimistic =\n     optimist.from(1)\n     |&gt; optimist.update(int.add(_, 1))\n     |&gt; optimist.unwrap\n\n   optimistic |&gt; should.equal(2)\n }\n \n\n ### Multiple optimistic updates\n\n \n import gleeunit/should\n import optimist\n\n pub fn example() {\n   let optimistic =\n     optimist.from(1)\n     |&gt; optimist.update(int.add(_, 1))\n     |&gt; optimist.update(int.add(_, 2))\n     |&gt; optimist.unwrap\n\n   optimistic |&gt; should.equal(3)\n }\n \n\n ### Rejecting multiple optimistic updates\n\n \n import gleeunit/should\n import optimist\n\n pub fn example() {\n   let optimistic =\n     optimist.from(1)\n     |&gt; optimist.update(int.add(_, 1))\n     |&gt; optimist.update(int.add(_, 2))\n     |&gt; optimist.revert\n     |&gt; optimist.unwrap\n\n   optimistic |&gt; should.equal(1)\n }\n \n\nSynonyms:\noptimist.update\noptimist update","url":"optimist.html#update"},{"doc":"optimist","title":"optimist","content":" When building user interfaces, we are often faced with the question of what\n to do while waiting for a response from a server. One common pattern is to\n update the ui with an optimistic value - one we expect to get back if an\n operation is successful - and then update the ui again once we get an actual\n response.\n\n The `Optimistic` type is a simple way to model this pattern. A value can\n either be fully resolved or it can be an optimistic update pending resolution.\n The functions in this module help you manage and query the state of an\n optimistic update.\n","url":"optimist.html"}]);